#include "display.h"

display create_display () {
  display result;

  if (SDL_Init (SDL_INIT_VIDEO) < 0) {
    printf ("Could not initialize SDL.\n");
    exit (EXIT_FAILURE);
  }

  SDL_GL_SetAttribute (SDL_GL_CONTEXT_MAJOR_VERSION, 3);
  SDL_GL_SetAttribute (SDL_GL_CONTEXT_MINOR_VERSION, 2);
  SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1);
  SDL_GL_SetAttribute (SDL_GL_DEPTH_SIZE, 24);

  result.w = DEFAULT_WINDOW_WIDTH;
  result.h = DEFAULT_WINDOW_HEIGHT;
  result.window = SDL_CreateWindow ("Project 4- sphere", SDL_WINDOWPOS_CENTERED,
				    SDL_WINDOWPOS_CENTERED, DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT,
				    SDL_WINDOW_OPENGL | SDL_WINDOW_SHOWN);
  if (!result.window) {
    printf ("Could not open window.\n");
    exit (EXIT_FAILURE);
  }
  result.context = SDL_GL_CreateContext (result.window);
  SDL_GL_SetSwapInterval (1);
  
  glEnable (GL_DEPTH_TEST);
  glDepthFunc (GL_LESS);
  glEnable (GL_CULL_FACE);
  glCullFace (GL_BACK); //be careful ?
  glFrontFace (GL_CCW);

  glewExperimental = GL_TRUE;
  glewInit ();
  
  exit_on_gl_failure ("OpenGL initialization error.\n");
  
  return result;
}

void exit_on_gl_failure (const char* details) {
  const GLenum val = glGetError ();
  if (val != GL_NO_ERROR) {
    fprintf (stderr, "%s : %s\n", details, gluErrorString (val));
    exit (EXIT_FAILURE);
  }
}

void clear_display () {
  glClearColor (0.0, 0.0, 0.0, 1.0);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

void flip_display (display d) {
  SDL_GL_SwapWindow (d.window);
}

void destroy_display (display d) {
  SDL_GL_DeleteContext (d.context);
  SDL_DestroyWindow (d.window);
  SDL_Quit ();
}

//this is copypasta. i have no shame
GLuint load_shader (const char* filename, GLenum shader_type) {
  GLuint shader_id = 0;
  FILE* file;
  long file_size = -1;
  char* glsl_source;

  if (NULL != (file = fopen(filename, "rb")) &&
      0 == fseek(file, 0, SEEK_END) &&
      -1 != (file_size = ftell(file)))
    {
      rewind(file);
    
      if (NULL != (glsl_source = (char*)malloc(file_size + 1)))
	{
	  if (file_size == (long)fread(glsl_source, sizeof(char), file_size, file))
	    {
	      glsl_source[file_size] = '\0';

	      if (0 != (shader_id = glCreateShader(shader_type)))
		{
		  glShaderSource(shader_id, 1, &glsl_source, NULL);
		  glCompileShader(shader_id);

#ifdef SHADER_DEBUG
		  printf ("%s compilation logs \n", filename);
		  char log[500];
		  log[499] = '\0';
		  int l = 0;
		  glGetShaderInfoLog (shader_id, 499, &l, log);
		  printf ("%s\n", log);
#endif
		  
		  exit_on_gl_failure ("Could not compile a shader.\n");
		}
	      else
		fprintf(stderr, "ERROR: Could not create a shader.\n");
	    }
	  else
	    fprintf(stderr, "ERROR: Could not read file %s\n", filename);

	  free(glsl_source);
	}
      else
	fprintf(stderr, "ERROR: Could not allocate %i bytes.\n", file_size);

      fclose(file);
    }
  else
    {
      if (NULL != file)
	fclose(file);
      fprintf(stderr, "ERROR: Could not open file %s\n", filename);
    }

  return shader_id;
}
