#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <SDL.h>

#include "events.h"
#include "utils.h"
#include "sphere.h"
#include "display.h"
#include "player.h"
#include "world.h"

int main () {
  srand (time (NULL));

  events ev = init_events ();
  display d = create_display ();
  player p = init_player (d);
  world w = init_world ();

  int playing = 1;
  Uint32 last_frame = 0;
  while (playing) {
    //events
    update_events (&ev);
    if (ev.quit)
      playing = 0;
    if (ev.key[SDL_SCANCODE_RIGHT])
      turn_horizontally (&p, 0.01);
    if (ev.key[SDL_SCANCODE_LEFT])
      turn_horizontally (&p, -0.01);
    if (ev.key[SDL_SCANCODE_UP])
      turn_vertically (&p, 0.01);
    if (ev.key[SDL_SCANCODE_DOWN])
      turn_vertically (&p, -0.01);
    if (ev.key[SDL_SCANCODE_W])
      forward (&p, 0.005);
    if (ev.key[SDL_SCANCODE_S])
      forward (&p, -0.005);

    clear_display ();
    draw_world (w, p);
    flip_display (d);

    //control framerate
    Uint32 current_frame = SDL_GetTicks ();
    if (current_frame - last_frame < 17)
      SDL_Delay (17 +last_frame - current_frame);
    last_frame = current_frame;
  }

  destroy_world (w);
  destroy_display (d);
  return EXIT_SUCCESS;
}
