#include "sphere.h"

vector3f stereo_of_hopf (vector3f angles) {
  return stereo_of_embedded (embedded_of_hopf (angles));
}

vector3f stereo_of_embedded (vector4f embedding) {
  vector3f result;
  result.x = 2.0f * embedding.x / (1.0f - embedding.t);
  result.y = 2.0f * embedding.y / (1.0f - embedding.t);
  result.z = 2.0f * embedding.z / (1.0f - embedding.t);
  return result;
}

vector3f hopf_of_embedded (vector4f embedding) {
  vector3f result;
  float aux = embedding.x * embedding.x + embedding.y * embedding.y;
  float aux_2 = sqrt ((1.0f/aux - 1.0f)/4.0f);
  result.x = 2.0f * atan (aux_2);
  result.z = atan (embedding.y / embedding.x);
  if (embedding.x < 0.0f) //be careful, atan is a meanie
    result.z += M_PI;
  float angle = atan (embedding.t / embedding.z);
  if (embedding.z < 0.0f)
    angle += M_PI;
  result.y = angle - result.z;
  //make sure everything is in intended range
  result.x = positive_fmod (result.x, M_PI);
  result.y = positive_fmod (result.y, 2.0f * M_PI);
  result.z = positive_fmod (result.z, 2.0f * M_PI);
  return result;
}

vector3f hopf_of_stereo (vector3f proj) {
  return hopf_of_embedded (embedded_of_stereo (proj));
}

vector4f embedded_of_hopf (vector3f angles) {
  vector4f result;
  float denominator = sqrt (1.0f + 4.0f * tan (angles.x/2.0f) * tan (angles.x/2.0f));
  result.x = cos (angles.z) / denominator;
  result.y = sin (angles.z) / denominator;
  result.z = 2.0f * cos (angles.y + angles.z) * tan (angles.x/2.0f) / denominator;
  result.t = 2.0f * sin (angles.y + angles.z) * tan (angles.x/2.0f) / denominator;
  return result;
}

vector4f embedded_of_stereo (vector3f proj) {
  vector4f result;
  float denominator = proj.x * proj.x + proj.y * proj.y + proj.z * proj.z + 4;
  result.x = 4.0f * proj.x / denominator;
  result.y = 4.0f * proj.y / denominator;
  result.z = 4.0f * proj.z / denominator;
  result.t = 1.0f - 8.0f / denominator;
  return result;
}

vector4f embedded_of_polar (vector3f angles) {
  vector4f result;
  float cy = cos (angles.y);
  float cz = cos (angles.z);
  result.t = cos (angles.x) * cy * cz;
  result.x = sin (angles.x) * cy * cz;
  result.y = sin (angles.y) * cz;
  result.z = sin (angles.z);
  return result;
}

vector3f polar_of_embedded (vector4f embedding) {
  vector3f result;
  result.z = asin (embedding.z);
  float cz = cos (result.z);
  result.y = asin (embedding.y / cz);
  float aux = asin (embedding.x / (cz * cos (result.y)));
  float aux_2 = (embedding.t > 0.0f) ? aux : M_PI - aux;
  result.x = (aux_2 < M_PI) ? aux_2 : aux_2 - 2.0f * M_PI;
  return result;
}

vector3f stereo_of_polar (vector3f angles) {
  return stereo_of_embedded (embedded_of_polar (angles));
}

vector3f polar_of_stereo (vector3f proj) {
  return polar_of_embedded (embedded_of_stereo (proj));
}
